from Bio import SeqIO
from os import listdir, path
from optparse import OptionParser
import copy

def GetFiles(path):
    files = listdir(path)
    filtered_files = [path + '/' + name for name in listdir(path) if name.endswith('.gbff')]
    return filtered_files

class Record:
    def __init__(self):
        self.gene = ''
        self.cds = ''

    def __str___(self):
        return str(self.gene) + str(self.cds)

    def __copy__(self):
        newone = type(self)()
        newone.__dict__.update(self.__dict__)
        return newone

def IsGood(name, masks):
    if (len(masks) == 0):
        return True
    for m in masks:
        if name.startswith(m):
            return True
    return False

def GetNameFromRecord(record, masks):
    strand = record.gene.location.strand

    if record.gene != '':
        if 'gene' in record.gene.qualifiers:
            name = record.gene.qualifiers['gene']
        elif record.cds != '':
            if 'gene' in record.cds.qualifiers:
                name = record.cds.qualifiers['gene']
            else:
                return ''
        else:
            return ''
    else:
        return ''
    name = name[0].replace('-','')

    if IsGood(name, masks):
        return ('+' if strand == 1 else '-') + name
    else:
        return ''

def read_masks(masks_file):
    with open(masks_file, 'r') as masks_input:
        return [m.strip() for m in masks_input.readlines()]

def get_sequences(path, masks, output_file):
    with open(output_file, 'w') as out:
        for f in GetFiles(path):
            with open(f, 'rU') as handle:
                for record in SeqIO.parse(handle, 'gb'):
                    if record.description.find('plasmid') != -1:
                        continue

                    top = 'C' if record.annotations['topology'] == 'circular' else 'L'
                    genes = []
                    for rec in record.features:
                        if (rec.type == 'gene'):
                            genes.append(Record())
                            genes[-1].gene = rec
                        elif (rec.type == 'CDS'):
                            genes[-1].cds = rec
                        else:
                            continue

                    names = []
                    for e in genes:
                        n = GetNameFromRecord(e, masks)
                        if (n != ''):
                            names.append(n)
                    out.write(record.description + '; 1; ' + top + str(len(names)) + ': ' + ''.join(names) + ';\n')

parser = OptionParser()
parser.add_option('-i', '--input_dir', dest='input', help='directory with *.gbff files')
parser.add_option('-m', '--masks', dest='masks', help='file with allowed gene masks, if empty - all genes are being considered')
parser.add_option('-o', '--output', dest='output', help='file with gene sequences')
(options, args) = parser.parse_args()
masks = read_masks(options.masks)
get_sequences(options.input, masks, options.output)
